#
# spec file for package keepalived
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


#Compat macro for new _fillupdir macro introduced in Nov 2017
%if ! %{defined _fillupdir}
  %define _fillupdir /var/adm/fillup-templates
%endif

%if 0%{?suse_version} > 1210
%bcond_without systemd
%else
%bcond_with    systemd
%endif
%bcond_with dbus
%bcond_with json

Name:           keepalived
Version:        1.4.1
Release:        lp150.2.1
Summary:        A keepalive facility for Linux
License:        GPL-2.0-or-later
Group:          Productivity/Networking/Routing
Url:            http://www.keepalived.org/
Source:         http://www.keepalived.org/software/%{name}-%{version}.tar.gz
Source2:        keepalive-rpmlintrc
Patch1:         keepalive-init.patch
# PATCH-FIX-UPSTREAM: https://github.com/acassen/keepalived/commit/947248af144bcab6376ccddab8dc40f313b14281.patch
Patch2:         linux-4.15.patch
BuildRequires:  libnfnetlink-devel
%if %{with json}
BuildRequires:  libjson-c-devel
%endif
BuildRequires:  net-snmp-devel
BuildRequires:  openssl-devel
BuildRequires:  pkgconfig
BuildRequires:  snmp-mibs
%if 0%{?suse_version} > 1110
BuildRequires:  pkgconfig(dbus-1)
BuildRequires:  pkgconfig(gio-2.0)
BuildRequires:  pkgconfig(json-c)
BuildRequires:  pkgconfig(libipset)
BuildRequires:  pkgconfig(libiptc)
BuildRequires:  pkgconfig(libnl-3.0)
BuildRequires:  pkgconfig(xtables)
%else
BuildRequires:  dbus-1-devel
BuildRequires:  glib2-devel
BuildRequires:  ipset-devel
BuildRequires:  libiptc-devel
BuildRequires:  libnl-devel
BuildRequires:  libxtables-devel
%endif
BuildRequires:  libnfnetlink-devel
# BuildRequires:  netsnmp-devel
BuildRequires:  popt-devel
%if 0%{?suse_version} && 0%{?suse_version} <= 1110
BuildRequires:  cyrus-sasl-devel
%endif
Requires(pre):  pwdutils
Requires(pre):  %fillup_prereq
%if %{with systemd}
BuildRequires:  systemd-rpm-macros
%{?systemd_requires}
%else
Requires(pre):  %insserv_prereq
%endif

BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
This project provides facilities for load balancing and high-availability to
Linux system and Linux-based infrastructures. The load-balancing framework
relies on the Linux Virtual Server (IPVS) kernel module providing Layer4 load
balancing.  Keepalived implements a set of checkers to dynamically and
adaptively maintain and manage loadbalanced server pool according their health.
High-availability is achieved by the VRRP protocol, a fundamental brick for
router failover. In addition, Keepalived implements a set of hooks to the VRRP
finite state machine, providing low-level and high-speed protocol interactions.
Keepalived frameworks can be used independently or all together to provide
resilient infrastructures.

%prep
%setup -q
%patch1 -p1
%patch2 -p1
chmod 644 doc/samples/*

%build
export STRIP=true
export CPPFLAGS="$(pkg-config --cflags libnfnetlink libiptc libipset xtables)"
export CFLAGS="%optflags -DOPENSSL_NO_SSL_INTERN"
%configure \
  --disable-silent-rules \
  %if %{with json}
  --enable-json \
  %endif
  --enable-snmp \
  --enable-snmp-vrrp \
  --enable-snmp-rfc \
  --enable-snmp-rfcv2 \
  --enable-snmp-rfcv3 \
  %if %{with dbus}
  --enable-dbus \
  --enable-dbus-create-instance \
  %endif
  %if %{with systemd}
  --with-init=systemd \
  --with-systemdsystemunitdir="%{_unitdir}" \
  %endif
  --enable-snmp-checker \
  --enable-snmp-rfc \
  --enable-snmp-rfcv2 \
  --enable-snmp-rfcv3 \
  --enable-sha1 \
  --enable-routes \
  --enable-libiptc \
  --enable-libipset \
  --disable-libipset-dynamic \
  --enable-libnl \
  --enable-stacktrace \
  --enable-json
make %{?_smp_mflags}

%install
%make_install
install -dD -m 0750 %{buildroot}%{_var}/lib/%{name}
install -D  -m 0644 %{buildroot}/etc/sysconfig/keepalived %{buildroot}%{_fillupdir}/sysconfig.%{name}

%if %{with systemd}
ln -s /sbin/service %{buildroot}%{_sbindir}/rckeepalived
%else
install -D -m 0750 keepalived/etc/init.d/keepalived.suse.init %{buildroot}/etc/init.d/keepalived
ln -s /etc/init.d/keepalived %{buildroot}%{_sbindir}/rckeepalived
%endif

chmod -R o= %{buildroot}/etc/keepalived
rm -rv %{buildroot}/etc/keepalived/samples/ %{buildroot}/etc/sysconfig/keepalived

%check
# A build could silently have LVS support disabled if the kernel includes can't
# be properly found, we need to avoid that.
if ! grep -q "#define _WITH_LVS_ *1" lib/config.h; then
    %{__echo} "ERROR: We do not want keepalived lacking LVS support." >&2
    exit 1
fi

%pre
getent group %{name} >/dev/null || /usr/sbin/groupadd -r %{name}
getent passwd %{name} >/dev/null || \
	/usr/sbin/useradd -g %{name} -s /bin/false -r -c "Keepalived" \
	-d %{_var}/lib/%{name} %{name}
%if %{with systemd}
%service_add_pre %{name}.service
%endif

%preun
%if %{with systemd}
%service_del_preun %{name}.service
%else
%stop_on_removal %{name}
%endif

%post
%fillup_only %{name}
%if %{with systemd}
%service_add_post %{name}.service
%endif

%postun
%if %{with systemd}
%service_del_postun %{name}.service
%else
%insserv_cleanup
%restart_on_update %{name}
%endif

%files
%defattr(-,root,root)
%doc AUTHOR ChangeLog CONTRIBUTORS COPYING README
%doc %{_datadir}/doc/keepalived/
%doc doc/samples/
%doc doc/keepalived.conf.SYNOPSIS doc/NOTE_vrrp_vmac.txt
%dir  %{_sysconfdir}/keepalived
%dir %attr(-,keepalived,keepalived) %{_var}/lib/%{name}
%{_fillupdir}/sysconfig.%{name}
%config(noreplace)  %{_sysconfdir}/keepalived/*conf
%{_bindir}/genhash
%{_sbindir}/rckeepalived
%{_sbindir}/keepalived
%{_mandir}/man1/genhash.1*
%{_mandir}/man5/keepalived.conf.5*
%{_mandir}/man8/keepalived.8*
%{_datadir}/snmp/mibs/KEEPALIVED-MIB.txt
%{_datadir}/snmp/mibs/VRRP-MIB.txt
%{_datadir}/snmp/mibs/VRRPv3-MIB.txt
#
%if %{with dbus}
%config /etc/dbus-1/system.d/org.keepalived.Vrrp1.conf
%{_datadir}/dbus-1/interfaces/org.keepalived.Vrrp1.Instance.xml
%{_datadir}/dbus-1/interfaces/org.keepalived.Vrrp1.Vrrp.xml
%endif
#
%if %{with systemd}
%{_unitdir}/%name.service
%else
/etc/init.d/keepalived
%endif

%changelog
* Mon Apr 30 2018 crrodriguez@opensuse.org
- Only Require insserv on distributions without systemd.
- Fix systemd related requires/buildRequires
- Do not run scriptlets that use insserv when using systemd
* Thu Feb 22 2018 dmueller@suse.com
- add linux-4.15.patch
* Wed Feb 21 2018 dmueller@suse.com
- update to 1.4.1:
  * Improve and fix use of getopt_long().
    We musn't use a long option val of 1, since getopt_long() can return
    that value.
    getopt_long() also returns longindex == 0 when there is no matching
    long option, and there needs to be careful checking if there is an
    error to work out whether a long or short option was used, which is
    needed for meaningful error messages.
  * Write assert() messages to syslog.
    assert()s are nasty things, but at least let's get the benefit of
    them, and write the messages to syslog, rather than losing them down
    stderr.
  * Enable sorry server at startup if quorum down due to alpha mode
    If alpha mode is configured on sufficient checkers so that a
    virtual server doesn't have a quorum, we need to add the sorry
    server at startup, otherwise it won't be added until a quorum has
    been achieved and subsequently lost again. In the case where some
    of the checkers remain in the down state at startup, this would have
    meant that the sorry server never got added.
  * For virtual servers, ensure quorum <= number of real servers
    If the quorum were gigher than the number of real servers, the
    quorum for the real server to come up could never be achieved, so
    if the quorum is greater than the number of real servers, reduce it
    to the number of real servers.
  * Fix some SNMP keepalived checker integer types and default values.
    Some virtual server and real server values were being sent to SNMP
    with a signed type whereas the value is unsigned, so set the type
    field correctly.
    Some virtual server and real server values that apply to checkers
    are set to nonsense default values in order to determine if a
    value has been specified. Handle these values when reporting them
    to SNMP replying with 0 rather than a nonsense value.
  * Fix some MALLOC/FREE issues with notify FIFOs.
  * Add instance_name/config_id to alert emails' subjects if configured.
    If multiple instances of keepalived are running, either different
    instance_names and/or config_ids, it is useful to know which
    keepalived instance the email relates to.
  * Ensure that email body string isn't unterminated.
    Using strncpy() needs to ensure that there is a nul termination byte,
    so this commits adds always writing a nul byte to the end of the buffer.
  * Remove duplicate fault notification.
  * Fix problem with scripts found via PATH with a '/' in parameters.
    Recent discussions on issue #101 led to discovering that if an
    executable without a fully qualified name was specified as a script
    and there was a '/' character in the parameters, then the path
    resolution would not work.
  * Send SNMP traps when go from backup to fault due to sync group.
    Commit 020a9ab added executing notify_fault for vrrp instances
    transitioning from backup to fault state due to another instance
    in the sync group going to fault state. This commit adds sending
    SNMP traps in the same circumstance.
  * Revert "Add instance_name/config_id to alert emails' subjects if
    configured". This should be handled by setting router_id
  * Add config option to send smtp-alerts to file rather than send emails
    This is useful for debugging purposes.
  * Add additional entry to Travis-CI build matrix.
  * Fix segfault if no sorry server configured for a virtual server.
* Mon Jan 22 2018 mrueckert@suse.de
- enable json stats and config dump support
  new BR: pkgconfig(json-c)
- disable dynamic loading of libipset and link it instead
- enable stacktrace support
- turn on snmp-rfcv2 and snmp-rfcv3 support
- do not reference the keepalived.socket in the rpm scriptlets
* Fri Jan 12 2018 lars@linux-schulserver.de
- update to 1.4.0
  * Add Linux build and runtime versions to -v output.
  * Log kernel version and build kernel version to log at startup.
  * Don't sleep for 1 send when exiting vrrp process if no vrrp instances.
  * With large configurations the syslog can get flooded and drop output.
    This commit adds options to not log to syslog, and also to log all
    output to files.
  * Add option to only flush log files before forking.
  * Don't poll netlink for all interfaces each time add a VMAC.
    We can poll for the individual interface details which significantly
    reduces what we have to process.
  * Print interface details in keepalived.data output.
  * Add high performace child finder code.
    The code to find the relevant thread to execute afer a child process
    (either a vrrp track script or a misc_check healthchecker) was doing
    a linear search for the matching pid, which if there are a large number
    of child processes running could become time consuming.
    The code now will enable high performance child finding, based on using
    mlists hashed by the pid, if there are 32 or more vrrp track scripts or
    misc check healthcheckers. The size of the mlist is based on the number
    of scripts, with a limit of 256.
  * Improve high performance child termination timeout code.
  * Preserve filename in script path name resolution.
    Some executables change their behaviour depending on the name by
    which they are invoked (e.g. /usr/sbin/pidof when it is a link to
    /usr/sbin/killall5). Using realpath() changes the file name part
    if it is a symbolic link. This commit resolves all symbolic links
    to directories, but leaves the file name part unaltered. It then
    checks the security of both the path to the link and the path to
    the real file.
  * Handle scripts names that are symbolic links properly.
  * Fix some RFC SNMP issues.
  * Fix removing left-over addresses if keepalived aborts.
  * Update openssl use to stop using deprecated functions
    openssl from version 1.1 deprecated certain functions that keepalived
    was using. This commit ceases using those functions if the version
    of openssl is >= 1.1.
  * Allow sync groups with only 1 member, but issue a warning.
  * Add replaceable parameters in configuration files.
  * Add multiline configuration definitions.
  * Fix keepalived.conf(5) man page.
  * Suppress error message when removing leftover addresses at startup.
  => find more changes at /usr/share/doc/packages/keepalived/
- rebase keepalive-init.patch
- use upstream systemd service file instead providing an own one
  => removed keepalived.service
- remove executable bit from samples in docdir
- check that LVS support is enabled
- optionally enable dump configuration and stats as JSON (via bcond)
  => BuildRequire libjson-c-devel
- restrict /etc/keepalived permissions to root
* Mon Nov 27 2017 jengelh@inai.de
- Do not suppress errors from useradd.
- Ensure neutrality of description.
* Mon Nov 27 2017 igarcia@suse.com
- update to 1.3.9:
  Revert using github tarball and use original source again.
  Too many fixes and features to list, refer to
  /usr/share/doc/packages/keepalived/ChangeLog for a detailed list.
* Thu Nov 23 2017 rbrown@suse.com
- Replace references to /var/adm/fillup-templates with new
  %%_fillupdir macro (boo#1069468)
* Thu Feb 16 2017 mrueckert@suse.de
- use tarball from https://github.com/acassen/keepalived/issues/524
  the original tarball did not build. This has the necessary fix
  applied. for the 1.3.4 update see the TODO entry in the preamble.
* Wed Feb 15 2017 mrueckert@suse.de
- update to 1.3.3
  Some minor fix, extensions and updates. snapcraft support. Refer
  to /usr/share/doc/packages/keepalived/ChangeLog for more infos.
* Mon Dec 12 2016 mrueckert@suse.de
- fix building with libnfnetlink. the additional include path needs
  to be in CPPFLAGS instead of CFLAGS now.
- enabled a few more features:
  - enhanced snmp support (V2/V3 RFC)
  - make sure we build with ipset/libiptc and routes support
- prepared dbus support: waiting for boo#1015141
* Mon Dec 12 2016 mrueckert@suse.de
- update 1.3.2
  - Security focused on notify heplers. Some minor fix and
    extensions.
  - changes from 1.3.1
  - Quick script fix for regression brought by last release.
  - changes from 1.3.0
  - New MAJOR release with stabilization fixes. Support to DBus.
    Conf extensions. Parser error log. Security extensions to run
    scripts more secure.
  - changes from 1.2.24
  - MAJOR release with stabilization fixes and new features like
    support to network namespace.
  Refer to /usr/share/doc/packages/keepalived/ChangeLog
  for more infos.
* Wed Jul 20 2016 michael@stroeder.com
- update to 1.2.23
  Some VRRP fixes. Some Healthcheckers fixes.
  Refer to ChangeLog for more infos.
* Fri Jul  8 2016 mrueckert@suse.de
- update to 1.2.22
  Some VRRP fixes. Refer to ChangeLog for more infos.
- update to 1.2.21
  Some fixes for last major release 1.2.20. Extensions on vrrp
  framework. Refer to ChangeLog for more infos.
- update to 1.2.20
  BUNCH of extensions, fixes, cleanup & production considerations.
  Distro packages maintainers are strongly encouraged to upgrade.
- new BR libnfnetlink-devel
- we no longer ship the VRRP-MIB
* Thu Feb 11 2016 lars@linux-schulserver.de
- enhanced keepalive-init.patch :
  + replace tabs with spaces
  + read /etc/sysconfig/keepalived, if exists and use the settings
    there instead of the default KEEPALIVED_OPTIONS in case the
    user changed them
* Thu Jan 28 2016 mrueckert@suse.de
- use package name buildrequires on sle11 to fix building
* Thu Jan 28 2016 mrueckert@suse.de
- enable snmp for better monitoring
- enable sha1 support
* Wed Oct  7 2015 dimstar@opensuse.org
- Update to version 1.2.19:
  + vrrp: fix checksum computation in vrrp v2 for socket family
    AF_INET.
  + Some cosmetics at Makefile stuff.
- Changes from version 1.2.18:
  + some cosmetics changes (in memory and parser).
  + remove dead/not used code.
  + revert notify script brought by last release.
  + revert VRRP preemption speed up extension.
  + vrrp: ix vrrp removes incorrect IPv4 address when VIPs are
    removed.
  + vrrp: Re-enable VRRPv2 checksum on inbound pkts.
- Changes from version 1.2.17:
  + zalloc use xalloc for consistency.
  + memory: fix wrong size calculation in zfree.
  + Fix keepalived snmp configuration.
  + Change comments to match kernel style.
  + smtp: Fix wrong algorithm in RCPT-TO building.
  + Lots of vrrp fixes.
- Changes from version 1.2.16:
  + Properly close netlink channel to avoid fd leak.
  + Use getaddrinfo instead of gethostbyname to workaround glibc
    gethostbyname function buffer overflow (boo#949238).
  + Lots of ipvs fixes.
* Wed Oct  7 2015 mrueckert@suse.de
- no longer install the init script on systemd systems
* Wed Mar 11 2015 dimstar@opensuse.org
- Update to version 1.2.15:
  + Bugfixes.
- Changes from version 1.2.14:
  + VRRP bugfixes and extensions. IPVS bugfixes and code code
    cleanup.
- Changes from version 1.2.13:
  + VRRP fixes and extensions. Extrend and unify checker
    framework.
* Mon Feb  2 2015 crrodriguez@opensuse.org
- Build with -DOPENSSL_NO_SSL_INTERN, if package starts accessing
  the SSL library internals it must fail to build now, in upcoming
  openSSL versions structures are opaque.
- BuildRequire libnl3
- Do not strip binaries, fix -debuginfo packages.
* Sun Nov  9 2014 Led <ledest@gmail.com>
- fix bashisms in pre script
* Thu Jul 31 2014 dimstar@opensuse.org
- Rename rpmlintrc to %%{name}-rpmlintrc.
  Follow the packaging guidelines.
* Tue Feb 11 2014 boris@steki.net
- updated to latest upstream version 1.2.12
  + Fix reallocation issue introduced in last merge.
  + Fix some minor memory leaks.
  + Better libnl support and selection.
  + VRRP unicast TTL fix.
  + Support to newer libnl.
  + More IPv6 support.
  + Fix/extend VRRP gratuitous ARP handling.
  + Support xmit VRRP packets from base VMAC interface.
  + VRRP multicast group tweaking.
  + Fixed VRRP socket sync while leaving FAULT state.
  + Code cleanup and cosmetics.
* Tue Jan  7 2014 speilicke@suse.com
- Add cyrus-sasl for old distros
* Tue Nov 19 2013 speilicke@suse.com
- Update to version 1.2.9:
  + Extended VRRP code for faster sync and transition.
  + Fixed VRRP unicast code to support routed packet.
  + Fixed VRRP checksum computation.
  + Extended VRRP code tweaking IPv6 VIP install by disabling DAD algo and setting deprecated flag.
  + Fixed some issues in checker framework while processing hysteresis.
  + Extended checker framework to support use of status_code and digest at a time.
- Changes from version 1.2.8:
  + Add support for VRRP unicast.
  + Add support for VRRP IPv6 routes.
  + Add support to LVS One-Packet Scheduling.
  + Add CLI core framework.
  + Misc bugfixes, typo and cosmetics.
- Drop keepalived_man_fix.patch: merged upstream
* Tue Nov 20 2012 mrueckert@suse.de
- initial package of 1.2.7
